package de.hdm_stuttgart.mi.sda1.exam.test.ignore;

import java.io.File;
import java.io.IOException;
import java.util.Optional;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jdom2.JDOMException;
import org.jdom2.ProcessingInstruction;
import org.jdom2.input.SAXBuilder;
import org.jdom2.input.sax.XMLReaders;

/**
 * Test and evaluate an individual XML instance.
 *
 */
public class InstanceTest {
   
   final SAXBuilder parser = new SAXBuilder(XMLReaders.XSDVALIDATING);
   
   final File instanceFilename;
   
   /**
    * An examinee may reach this number of point. The value is being defined in
    * the corresponding XML instance's PI like e.g.:
    * 
    * &lt;?xmlTest
    points = "2" ... ?&gt;
    *   
    */
   public final int reachablePoints;
   
   /**
    * Is this instance expected to be valid or not? The value is being defined in
    * the corresponding XML instance's PI like e.g.:
    * 
    * &lt;?xmlTest expectedToBeValid= "false" ... ?&gt;
    */
   public final boolean expectedToBeValid;
   /**
    * Does the current instance test succeed?
    */
   public final boolean testSucceeded;
   
   /**
    * Error message in case of unexpected behavior e.g.
    * when an invalid instance is supposed to be valid.
    */
   public final Optional<String> errMsg;
   
   private static Logger log = LogManager.getLogger(InstanceTest.class);

   @Override
   public String toString() {
      if (testSucceeded) {
         return instanceFilename + ": " + reachablePoints + " point" + (1 == reachablePoints ? "" : "s") ;
      } else {
         return instanceFilename + ": " + errMsg.get() + ",\n missing" + reachablePoints +" point(s)";
      }
   }

   /**
    * Testing an individual instance to be either valid or invalid.
    * 
    * @param xmlTest The PI meta annotation header containing the expected
    *        result, points etc.
    * @param xmlTestFileDir The directory containing the XML instance files.
    * @param xmlInstance The current instance to be evaluated.
    */
   public InstanceTest(final ProcessingInstruction xmlTest,
         final String xmlTestFileDir, final File xmlInstance) {
      
      this.instanceFilename = xmlInstance;
      
      if (null == xmlTest.getPseudoAttributeValue("points")) {
         log.fatal("Mandatory <?xmlTest points='...' ... ?> is missing in file '"
      + xmlInstance + "'");
         System.exit(1);
      }
      reachablePoints = Integer.parseInt(xmlTest.getPseudoAttributeValue("points"));
      
      if (null == xmlTest.getPseudoAttributeValue("expectedToBeValid")) {
         log.fatal("Mandatory <?xmlTest expectedToBeValid='true|false' ... ?> is missing in file '" + xmlInstance + "'");
         System.exit(1);
      }
      expectedToBeValid = Boolean.parseBoolean(xmlTest.getPseudoAttributeValue("expectedToBeValid"));
      
      final Optional<String> preconditionValidFilename;

      boolean preconditionSucceeded = false;
      String tmpPreconditionErrMsg = null;
      
      if (null == xmlTest.getPseudoAttributeValue("preconditionValid")) {
    	  preconditionValidFilename = Optional.empty();
      } else {
    	  preconditionValidFilename = Optional.of(xmlTestFileDir + File.separator + 
    		      xmlTest.getPseudoAttributeValue("preconditionValid"));
          try {
              parser.build(preconditionValidFilename.get());
              log.info("Precondition file '" + preconditionValidFilename.get() + "' is valid");
              preconditionSucceeded = true;            
           } catch (final JDOMException | IOException e) {
              log.info("Precondition file '" + preconditionValidFilename.get() + "' is invalid or missing:", e);
              tmpPreconditionErrMsg = "File '" + preconditionValidFilename.get() + "' is either invalid or missing";
           }
      }

      if (!preconditionValidFilename.isPresent() || preconditionSucceeded) {
         boolean tmpInstanceIsValid = false;
         try {
         parser.build(xmlInstance);
         tmpInstanceIsValid = true;
         } catch (final JDOMException | IOException e) {
            log.info("Instance file '" + xmlInstance + "' is invalid:", e);
         }
         testSucceeded =  expectedToBeValid == tmpInstanceIsValid; 
         if (testSucceeded) {
            errMsg = Optional.empty();
         } else if (testSucceeded) {
            errMsg = Optional.of("Instance '" + xmlInstance +
            		"' is expected to be invalid!");
         } else {
            errMsg = Optional.of("Instance '" + xmlInstance +
            		"' is expected to be valid!");
         }         
      } else {
          testSucceeded = false;
         errMsg = Optional.of(tmpPreconditionErrMsg);
         
      }
   }
}
